import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_counts_by_hour_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/counts_by_hour.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/counts_by_hour.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardCountsByHour extends StatelessWidget {
  const CardCountsByHour({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getDistributionDaysCount();

    return BlocBuilder<DataCountsByHourCubit, DataCountsByHourState>(
      builder: (BuildContext context, DataCountsByHourState data) {
        final CountsByHourData counts = data.countsByHour ?? CountsByHourData.fromJson({});

        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'counts_by_hour'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: update(daysCount),
          content: ChartCountsByHour(chartData: counts),
        );
      },
    );
  }

  Widget update(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<CountsByHourData> future = ScrobblesApi.fetchCountsByHour(daysCount);

    return BlocBuilder<DataCountsByHourCubit, DataCountsByHourState>(
      builder: (BuildContext context, DataCountsByHourState data) {
        return FutureBuilder<CountsByHourData>(
          future: future,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataCountsByHourCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
