import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_discoveries_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/discoveries.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/discoveries_artists.dart';
import 'package:scrobbles/ui/widgets/charts/discoveries_tracks.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardDiscoveries extends StatelessWidget {
  const CardDiscoveries({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getDiscoveriesDaysCount();

    return BlocBuilder<DataDiscoveriesCubit, DataDiscoveriesState>(
      builder: (BuildContext context, DataDiscoveriesState data) {
        final TextTheme textTheme = Theme.of(context).primaryTextTheme;

        final DiscoveriesData discoveries = data.discoveries ?? DiscoveriesData.fromJson({});

        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'discoveries_title'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: update(daysCount),
          content: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text(
                'discoveries_artists_title',
                style: textTheme.titleMedium!.apply(fontWeightDelta: 2),
              ).tr(),
              const SizedBox(height: 8),
              ChartDiscoveriesArtists(chartData: discoveries),
              const SizedBox(height: 8),
              Text(
                'discoveries_tracks_title',
                style: textTheme.titleMedium!.apply(fontWeightDelta: 2),
              ).tr(),
              const SizedBox(height: 8),
              ChartDiscoveriesTracks(chartData: discoveries),
            ],
          ),
        );
      },
    );
  }

  Widget update(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<DiscoveriesData> future = ScrobblesApi.fetchDiscoveries(daysCount);

    return BlocBuilder<DataDiscoveriesCubit, DataDiscoveriesState>(
      builder: (BuildContext context, DataDiscoveriesState data) {
        return FutureBuilder<DiscoveriesData>(
          future: future,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataDiscoveriesCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
