import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_heatmap_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/models/heatmap.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/heatmap.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardHeatmap extends StatelessWidget {
  const CardHeatmap({super.key});

  @override
  Widget build(BuildContext context) {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    final int daysCount = settings.getDistributionDaysCount();

    return BlocBuilder<DataHeatmapCubit, DataHeatmapState>(
      builder: (BuildContext context, DataHeatmapState data) {
        final HeatmapData heatmap = data.heatmap ?? HeatmapData.fromJson({});

        return CardContent(
          color: Theme.of(context).colorScheme.surface,
          title: 'heatmap'.tr(
            namedArgs: {
              'daysCount': daysCount.toString(),
            },
          ),
          loader: update(daysCount),
          content: ChartHeatmap(chartData: heatmap),
        );
      },
    );
  }

  Widget update(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<HeatmapData> future = ScrobblesApi.fetchHeatmap(daysCount);

    return BlocBuilder<DataHeatmapCubit, DataHeatmapState>(
      builder: (BuildContext context, DataHeatmapState data) {
        return FutureBuilder<HeatmapData>(
          future: future,
          builder: (context, snapshot) {
            if (snapshot.hasError) {
              return ShowErrorWidget(message: '${snapshot.error}');
            }

            BlocProvider.of<DataHeatmapCubit>(context).update(snapshot.data);

            return !snapshot.hasData ? loading : done;
          },
        );
      },
    );
  }
}
