import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import 'package:scrobbles/config/app_colors.dart';
import 'package:scrobbles/models/timeline.dart';
import 'package:scrobbles/ui/widgets/abstracts/custom_bar_chart.dart';

class ChartTimelineCounts extends CustomBarChart {
  final TimelineData chartData;

  const ChartTimelineCounts({super.key, required this.chartData});

  final double verticalTicksInterval = 50;

  @override
  Widget build(BuildContext context) {
    if (this.chartData.data.keys.length == 0) {
      return SizedBox(
        height: this.chartHeight,
      );
    }

    return Container(
      height: this.chartHeight,
      child: LayoutBuilder(
        builder: (context, constraints) {
          final double maxWidth = constraints.maxWidth;
          final int barsCount = this.chartData.data.keys.length;

          return getBarChart(
            barWidth: this.getBarWidth(maxWidth, barsCount),
            backgroundColor: Theme.of(context).colorScheme.onSurface,
          );
        },
      ),
    );
  }

  double getMaxCountsValue() {
    double maxValue = 0;

    this.chartData.data.keys.forEach((key) {
      TimelineDataValue? value = this.chartData.data[key];
      if (value != null) {
        double counts = value.counts.toDouble();
        if (counts > maxValue) {
          maxValue = counts;
        }
      }
    });

    return maxValue;
  }

  List<BarChartGroupData> getDataCounts(double barWidth) {
    List<BarChartGroupData> data = [];

    final barColor = AppColors.contentColorOrange;

    this.chartData.data.keys.forEach((key) {
      TimelineDataValue? value = this.chartData.data[key];
      if (value != null) {
        final int date = DateTime.parse(key).millisecondsSinceEpoch;
        final double counts = value.counts.toDouble();

        data.add(this.getBarItem(
          x: date,
          values: [counts],
          barColors: [barColor],
          barWidth: barWidth,
        ));
      }
    });

    return data;
  }

  Widget getVerticalLeftTitlesWidget(double value, TitleMeta meta) {
    return getVerticalTitlesSpacerWidget(value, meta);
  }
}
