import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import 'package:scrobbles/config/app_colors.dart';
import 'package:scrobbles/models/timeline.dart';
import 'package:scrobbles/ui/widgets/abstracts/custom_line_chart.dart';
import 'package:scrobbles/utils/color_extensions.dart';

class ChartTimelineEclecticism extends CustomLineChart {
  final TimelineData chartData;

  const ChartTimelineEclecticism({super.key, required this.chartData});

  final String verticalAxisTitleSuffix = '%';

  @override
  Widget build(BuildContext context) {
    if (this.chartData.data.keys.length == 0) {
      return SizedBox(
        height: this.chartHeight,
      );
    }

    final horizontalScale = getHorizontalScaleFromDates(this.chartData.data.keys);

    // Left/right margins: 12h, in milliseconds
    const double margin = 12 * 60 * 60 * 1000;

    return Container(
      height: this.chartHeight,
      child: LineChart(
        LineChartData(
          lineBarsData: getDataEclecticism(),
          borderData: noBorderData,
          gridData: noGridData,
          titlesData: getTitlesData(),
          lineTouchData: const LineTouchData(enabled: false),
          minX: (horizontalScale['min'] ?? 0) - margin,
          maxX: (horizontalScale['max'] ?? 0) + margin,
          maxY: 100,
          minY: 0,
        ),
        duration: const Duration(milliseconds: 250),
      ),
    );
  }

  List<LineChartBarData> getDataEclecticism() {
    List<FlSpot> spots = [];

    this.chartData.data.keys.forEach((element) {
      TimelineDataValue? value = this.chartData.data[element];
      if (value != null) {
        final double date = DateTime.parse(element).millisecondsSinceEpoch.toDouble();
        final double eclecticism = value.eclecticism.toDouble();

        spots.add(FlSpot(date, eclecticism));
      }
    });

    final baseColor = AppColors.contentColorCyan;
    final borderColor = baseColor.darken(20);

    return [
      LineChartBarData(
        isCurved: true,
        color: borderColor,
        barWidth: 2,
        isStrokeCapRound: false,
        dotData: const FlDotData(show: false),
        spots: spots,
      ),
    ];
  }

  Widget getVerticalRightTitlesWidget(double value, TitleMeta meta) {
    return getVerticalTitlesSpacerWidget(value, meta);
  }
}
